%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Sample code to reproduce the results for our dataset
%   in the following paper.
%
%   Yusuke Monno, Daisuke Kiku, Masayuki Tanaka, and Masatoshi Okutomi,
%   "Adaptive Residual Interpolation for Color and Multispectral 
%    Image Demosaicking,"
%   Sensors, vol.17, no.12, pp.2787-1-21, 2017.
%
% -------------------------------------------------------------------------
%   Copyright (C) 2017 Yusuke Monno and Daisuke Kiku. All rights reserved.
%   ymonno@ok.sc.e.titech.ac.jp 
%   http://www.ok.sc.e.titech.ac.jp/~ymonno/
%
%   Ver. 1.1 (minor bug fixed)
%   December 23, 2017..
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Please download the CAVE dataset from the following link.
%   http://www.cs.columbia.edu/CAVE/databases/multispectral/
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
addpath('Inputs/balloons_ms',genpath('Functions'),'SpectralData');

% spectral dimension (31 band)
dim = 31;
% number of spectral band
n = 5;
% gamma value for visualization
gam = 2.2;

% image name
imagename = 'balloons';

% load spectral reflectance data (31 band)
img = [];
for m = 1:dim
    tempimg = imread(sprintf('%s_ms_%02d.png',imagename,m));
    img = cat(3, img, tempimg(:,:,1));
end
img = im2double(img);
% size of spectral reflectance data
[h,w,s] = size(img);
% convert to vector form (31 x all pixels)
r = im2vec(img);

% illumination (D65)
load('CIE_D65_400_700_10.mat');
% diagonal illumination matrix with normalization
L = diag(CIE_D65/max(CIE_D65));
% rgb2xyz matrix with correct white point (0.3127 0.3290)
param=[0.3127 0.3290 0.64 0.33 0.30 0.60 0.15 0.06 1];
M = mat_rgb2xyz(param);

% % illumination (A)
% load('CIE_A_400_700_10.mat');
% % diagonal illumination matrix with normalization
% L = diag(CIE_A/max(CIE_A));
% % rgb2xyz matrix with correct white point (0.4476 0.4075)
% param=[0.4476 0.4075 0.64 0.33 0.30 0.60 0.15 0.06 1];
% M = mat_rgb2xyz(param);

% % illumination (F12)
% load('CIE_F12_400_700_10.mat');
% % diagonal illumination matrix with normalization
% L = diag(CIE_F12/max(CIE_F12));
% % rgb2xyz matrix with correct white point (0.4370 0.4044)
% param=[0.4370 0.4044 0.64 0.33 0.30 0.60 0.15 0.06 1];
% M = mat_rgb2xyz(param);

% ground-truth sRGB image
% xyz values by color matiching functions
load('CIE_2degCMF_400_700_10.mat');
XYZ = ref2XYZ(r,cmf,L,inv(M));
% xyz to sRGB(vector form) conversion with correct white point
sRGB = XYZ2sRGB(XYZ,inv(M));
% convert to image form
sRGB = reshape(sRGB', h, w, 3);
% gamma correction for only visualization
sRGB_gamma = clip(sRGB,0,1).^(1/gam);
% save ground-truth sRGB image
imwrite(sRGB_gamma,'sample_sRGB.bmp');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Single-sensor 5-band imaging
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% camera sensor sensitivity (simulated Gaussian)
load('C_uniform5_400_700_10.mat');
% camera sensor sensitivity (our prototype camera)
% load('C_emuda5_400_700_10.mat');

% system matrix (illumination x sensitivity)
H = C_uniform5' * L;

% ground-truth 5-band images
% vector form (5 x all pixels)
g = H * r;
% image form (height x width x band)
g = reshape(g', h, w, n);
% gamma correction for only visualization
g_gamma = g.^(1/gam);
% save ground-truth 5-band images
imwrite(g_gamma(:,:,1),'sample_B.bmp');
imwrite(g_gamma(:,:,2),'sample_C.bmp');
imwrite(g_gamma(:,:,3),'sample_G.bmp');
imwrite(g_gamma(:,:,4),'sample_O.bmp');
imwrite(g_gamma(:,:,5),'sample_R.bmp');

% ground-truth 5-band image
B = g(:,:,1);
C = g(:,:,2);
G = g(:,:,3);
O = g(:,:,4);
R = g(:,:,5);

% mosaicking
mosB = Bmosaic(B);
mosC = Cmosaic(C);
mosG = Gmosaic(G);
mosO = Omosaic(O);
mosR = Rmosaic(R);

% % parameters for the guided filter in demosaicking
% r = 4;
% eps = 0.00001;
% alpha = 3000;

% demosaicking
% [demR,demO,demG,demC,demB] = WGFdemosaicking(mosR,mosO,mosG,mosC,mosB,r,eps,alpha);
[demR,demO,demG,demC,demB] = ARIdemosaicking(mosR,mosO,mosG,mosC,mosB);

% the case of without mosaicking and demosaicking
% remove this if you include mosaicking and demosaicking process
% demB = B;
% demC = C;
% demG = G;
% demO = O;
% demR = R;

% gamma correction for only visualization
demB_gamma = demB.^(1/gam);
demC_gamma = demC.^(1/gam);
demG_gamma = demG.^(1/gam);
demO_gamma = demO.^(1/gam);
demR_gamma = demR.^(1/gam);

% save demosaicked 5-band images
imwrite(demB_gamma,'sample_demB.bmp');
imwrite(demC_gamma,'sample_demC.bmp');
imwrite(demG_gamma,'sample_demG.bmp');
imwrite(demO_gamma,'sample_demO.bmp');
imwrite(demR_gamma,'sample_demR.bmp');

% spectral estimation from demosaicked 5-band images
% regularization paremeter
reg = 0.1;
% spectral basis functions
load('SpectralBasis_400_700_10.mat');
% spectral estimation based on linear spectral model
es_r = es_ref5(demR,demO,demG,demC,demB,H,basis(:,1:8),reg);

% convert to sRGB image
% xyz values based on estimated spectral reflectance
demXYZ = ref2XYZ(es_r,cmf,L,inv(M));
% xyz to sRGB(vector form) conversion with correct white point
demsRGB = XYZ2sRGB(demXYZ,inv(M));
% convert to image form
demsRGB = reshape(demsRGB', h, w, 3);
% gamma correction for only visualization
demsRGB_gamma = clip(demsRGB,0,1).^(1/gam);
% save estimated sRGB image
imwrite(demsRGB_gamma,'sample_demsRGB.bmp');

% psnr calculation (5 band)
p(1,1)  = psnr(trim(R,10),trim(demR,10));
p(1,2)  = psnr(trim(O,10),trim(demO,10));
p(1,3)  = psnr(trim(G,10),trim(demG,10));
p(1,4)  = psnr(trim(C,10),trim(demC,10));
p(1,5)  = psnr(trim(B,10),trim(demB,10));
p(1,6)  = mean(p(1,1:5));

% psnr calculation (sRGB)
p(1,7)  = psnr(trim(sRGB(:,:,1),10),trim(demsRGB(:,:,1),10));
p(1,8)  = psnr(trim(sRGB(:,:,2),10),trim(demsRGB(:,:,2),10));
p(1,9)  = psnr(trim(sRGB(:,:,3),10),trim(demsRGB(:,:,3),10));
p(1,10) = mean(p(1,7:9));
p(1,11) = cpsnr(trim(sRGB,10),trim(demsRGB,10));

% convert sRGB to Lab
Lab = XYZ2Lab(sRGB2XYZ(trim(sRGB,10),M));
demLab = XYZ2Lab(sRGB2XYZ(trim(demsRGB,10),M));

% calculate deltaE
Eab = calcEab(Lab,demLab);
p(1,12) = mean(Eab(:));

% calculate CIEDE2000
s2 = size(Lab);
Labstd = reshape(Lab,s2(1)*s2(2),s2(3));
Labsample = reshape(demLab,s2(1)*s2(2),s2(3));
de00 = deltaE2000(Labstd,Labsample);
p(1,13) = mean(de00)


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % 3-band color imaging as reference
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % camera sensor sensitivity (3band)
% load('C_real3_400_700_10.mat');
% 
% % In the TIP paper, we used spectral sensitivities of a real RGB camera
% % that we measured.
% % You can also use publicly available camera sensitivity data from
% % http://www.cis.rit.edu/~dxl5849/projects/camspec/.
% 
% % system matrix (illumination x sensitivity)
% H = C_real3' * L;
% 
% % ground-truth 3-band images
% % vector form (3 x all pixels)
% g = H * r;
% % image form (height x width x 3)
% g = reshape(g', h, w, 3);
% % gamma correction for only visualization
% g_gamma = g.^(1/gam);
% % save ground-truth 3-band images
% imwrite(g_gamma(:,:,1),'sample_B.bmp');
% imwrite(g_gamma(:,:,2),'sample_G.bmp');
% imwrite(g_gamma(:,:,3),'sample_R.bmp');
% 
% % ground-truth 3-band image
% B = g(:,:,1);
% G = g(:,:,2);
% R = g(:,:,3);
% 
% % % mosaicking and demosaicking
% % % please include the Bayer mosaicking and demosaicking method here
% % [demR,demG,demB] = demosaic(R,G,B);
% 
% % the case of without mosaicking and demosaicking
% % remove this if you include mosaicking and demosaicking process
% demB = B;
% demG = G;
% demR = R;
% 
% % gamma correction for only visualization
% demB_gamma = demB.^(1/gam);
% demG_gamma = demG.^(1/gam);
% demR_gamma = demR.^(1/gam);
% 
% % save demosaicked 3-band images
% imwrite(demB_gamma,'sample_demB.bmp');
% imwrite(demG_gamma,'sample_demG.bmp');
% imwrite(demR_gamma,'sample_demR.bmp');
% 
% % spectral estimation from demosaicked 3-band images
% % regularization paremeter
% reg = 0.1;
% % spectral basis functions
% load('SpectralBasis_400_700_10.mat');
% % spectral estimation based on linear spectral model
% es_r = es_ref3(demR,demG,demB,H,basis(:,1:8),reg);
% 
% % convert to sRGB image
% % xyz values based on estimated spectral reflectance
% demXYZ = ref2XYZ(es_r,cmf,L,inv(M));
% % xyz to sRGB (vector form) conversion with correct white point
% demsRGB = XYZ2sRGB(demXYZ,inv(M));
% % convert to image form
% demsRGB = reshape(demsRGB', h, w, 3);
% % gamma correction for only visualization
% demsRGB_gamma = clip(demsRGB,0,1).^(1/gam);
% % save estimated sRGB image
% imwrite(demsRGB_gamma,'sample_demsRGB.bmp');
% 
% % psnr calculation (3 band)
% p(1,1)  = psnr(trim(R,10),trim(demR,10));
% p(1,2)  = psnr(trim(G,10),trim(demG,10));
% p(1,3)  = psnr(trim(B,10),trim(demB,10));
% p(1,4)  = mean(p(1,1:3));
% 
% % psnr calculation (sRGB)
% p(1,5)  = psnr(trim(sRGB(:,:,1),10),trim(demsRGB(:,:,1),10));
% p(1,6)  = psnr(trim(sRGB(:,:,2),10),trim(demsRGB(:,:,2),10));
% p(1,7)  = psnr(trim(sRGB(:,:,3),10),trim(demsRGB(:,:,3),10));
% p(1,8) = mean(p(1,5:7));
% p(1,9) = cpsnr(trim(sRGB,10),trim(demsRGB,10));
% 
% % convert sRGB to Lab
% Lab = XYZ2Lab(sRGB2XYZ(trim(sRGB,10),M));
% demLab = XYZ2Lab(sRGB2XYZ(trim(demsRGB,10),M));
% 
% % calculate deltaE
% Eab = calcEab(Lab,demLab);
% p(1,10) = mean(Eab(:));
% 
% % calculate CIEDE2000
% s2 = size(Lab);
% Labstd = reshape(Lab,s2(1)*s2(2),s2(3));
% Labsample = reshape(demLab,s2(1)*s2(2),s2(3));
% de00 = deltaE2000(Labstd,Labsample);
% p(1,11) = mean(de00)

